from pymongo import MongoClient
from datetime import datetime
from bson import ObjectId


class DemandePubDao:
    client = MongoClient('localhost', 27017)

    ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
    base_donnees = client.squelette

    ## Crétion d'une demande de publication
    # @param idCOntenu l'id du contenu d'origine
    # @param auteur l'auteur du contenu d'origine
    # @param idContenuEnCours l'id du contenu en cours
    # @param auteurEnCours l'auteur du contenu en cours

    def creer_demande_publication(self, idContenu, idContenuEnCours, auteurEnCours, auteur):

        self.base_donnees.demandesPub.insert_one(
            {"date": datetime.now().strftime("%d/%m/%y"), "auteurDemande": auteurEnCours,
             "auteurContenu": auteur,
             "idContenu": ObjectId(idContenu), "idContenuEnCours": ObjectId(idContenuEnCours), "etat": "enCours"})

    '''
    ## Fonction permettant de gérer une demande de publication 
    # @param idPublication l'id de la demande de publication
    # @param reponse la réponse donnée par l'auteur d'origine
    # @param auteur l'auteur d'origine
    def gerer_demande_pub(self, idPublication, reponse, auteur):
        if reponse == 'o':
            print("Publication acceptée")
            id_en_cours = Utils.get_id_contenu_en_cours_associe_a_une_publication(idPublication)
            branche = Utils.get_branche_contenu_en_cours(id_en_cours)
            idContenu = Utils.get_id_contenu_associe_a_une_publication(idPublication)
            chemin = Utils.get_chemin_du_contenu(idContenu)
            Utils.maj_contenu(chemin, idContenu, branche)
            Utils.statdb.demandesPub.update_one({"id": int(idPublication)}, {"$set": {"etat": "accepté"}})
            Utils.maj_tiers(idContenu, Utils.get_version_courante(idContenu), auteur)
            Utils.supprimer_contenu_en_cours(id_en_cours)
        else:
            print("Publication refusée ", idPublication)

            # on change la valeur "acceptation" de la demande de publication
            Utils.statdb.demandesPub.update_one({"id": int(idPublication)}, {"$set": {"etat": "refus"}})
            # Utils.statdb.demandesPub.update_one({"id": 1}, {"$set": {"etat": "refus"}})
    '''

    ## Supprimer une demande de publication
    # @param idPub l'id de la demande de publication

    def supprimer_publication(self,idPub):
        self.base_donnees.demandesPub.delete_one({"id": idPub})

    ## Récupérer l'auteur d'une demande de publication
    # @param idPub l'id de la demande de publication
    def get_auteur_demande_publication(self,idPub):
        return self.base_donnees.demandesPub.find_one({"id": idPub})["auteurDemande"]

    ## Récupérer lid du contenu (perso ou tiers) associé à une demande de publication
    # @param idDdePublication l'id de la demande de publication
    def get_id_contenu_associe_a_une_publication(self, idDdePublication):
        # print("L'id en cours que j'ai est", idContenuEnCours, " - ", type(idContenuEnCours))
        return self.base_donnees.demandesPub.find_one({"id": int(idDdePublication)})["idContenu"]

    ## Récupérer lid du contenu en cours associé à une demande de publication
    # @param idDdePublication l'id de la demande de publication
    def get_id_contenu_en_cours_associe_a_une_publication(self,idDdePublication):
        # print("L'id en cours que j'ai est", idDdePublication, " - ", type(idDdePublication))
        return self.base_donnees.demandesPub.find_one({"id": int(idDdePublication)})["idContenuEnCours"]

    ## Récupérer l'id max des demandes de publication (sera supprimé par la suite et remplacé par les id mongo)
    def get_max_id_demandes_de_publication(self):
        ## Fonction permettant de récupérer l'id max présent en base (pour pouvoir incrémenter un nouveau contenu).
        element = self.base_donnees.demandesPub.find().sort("id", -1).limit(1)

        for elt in element:
            # print("id : ", elt)
            return int(elt["id"])

    ## Récupérer les demandes de publication
    # @param auteur l'auteur des demandes
    # @param emises si true, récupère les demandes emises; les demandes recues sinon
    # @return les demandes de publications idoines
    def get_demandes_pub(self, auteur, emises=True):
        if emises:
            return self.base_donnees.demandesPub.find({"auteurDemande": auteur})
        else:
            return self.base_donnees.demandesPub.find({"auteurContenu": auteur})